/*=====================================================================
  File:      CSharpLangType.cs

  Summary:   Brief summary of the file contents and purpose.

---------------------------------------------------------------------
  This file is part of the Microsoft NGWS SDK Code Samples.

  Copyright (C) 2000 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

namespace LangUtil {

using System;
using System.Text;
using System.Reflection;
using System.Collections;

   public class CSharpLangType : LangType 
   {
      // constructor calls base constructor, passing Type object, then further processing.
      public CSharpLangType( Type ThisType ) : base( ThisType )
      {      
      }
   
      // constructor calls base constructor, passing ParameterInfo object, then further processing.
      public CSharpLangType( ParameterInfo ParamInfo ) : base( ParamInfo )
      {      
      }
            
      override public string TranslateTypeName( string strName )
      {
         // Strip the brackets here, if any
         bool bBrackets = false;
         if ( strName.Substring( strName.Length - 2 ) == "[]" )
         {
            bBrackets = true;
            strName = strName.Substring( 0, strName.Length - 2 );
         }

         // Translate the class name
         if ( strName == "Void" )
            strName = "void";
         else if ( strName == "Int16" )
            strName = "short";
         else if ( strName == "Int32" )
            strName = "int";
         else if ( strName == "Int64" )
            strName = "long";
         else if ( strName == "UInt16" )
            strName = "ushort";
         else if ( strName == "UInt32" )
            strName = "uint";
         else if ( strName == "UInt64" )
            strName = "ulong";
         else if ( strName == "Boolean" )
            strName = "bool";
         else if ( strName == "Char" )
            strName = "char";
         else if ( strName == "Byte" )
            strName = "byte";
         else if ( strName == "Single" )
            strName = "float";
         else if ( strName == "Double" )
            strName = "double";
         else if ( strName == "Decimal" )
            strName = "decimal";
         else if ( strName == "String" )
             strName = "string";
         else if ( strName == "Object" )
            strName = "object";    
            
         // Finally, append the brackets
         if ( bBrackets )
            strName += "[]";
         
         return strName;
      }      

      public CSharpConstructorInfo[] OrderedLangConstructors
      {
         get
         {
            ConstructorInfo[] Constructors = null;
            CSharpConstructorInfo[] CSharpConstructors = null;
            try
            {
               Constructors = m_TypeObject.GetConstructors( BindingFlags.NonPublic );
               CSharpConstructors = new CSharpConstructorInfo[ Constructors.Length ];
               ConstructorComparer ICompConst = new ConstructorComparer();
               Array.Sort( Constructors, ICompConst );
               for ( int i = 0; i < Constructors.Length; i++ )
                  CSharpConstructors[i] = new CSharpConstructorInfo( Constructors[i] );
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in CSharpLangType.OrderedConstructors.", Ex );
            }
            return CSharpConstructors;
         }
      }

      public CSharpMethodInfo[] OrderedLangMethods
      {
         get
         {
            MethodInfo[] Methods = null;
            CSharpMethodInfo[] CSharpMethods = null;
            try
            {
               Methods = m_TypeObject.GetMethods( BindingFlags.NonPublic );
               CSharpMethods = new CSharpMethodInfo[ Methods.Length ];
               // Order the member categories
               MethodComparer ICompMet = new MethodComparer();
               Array.Sort( Methods, ICompMet );
               for ( int i = 0; i < Methods.Length; i++ )
                  CSharpMethods[i] = new CSharpMethodInfo( Methods[i] );
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in CSharpLangType.OrderedConstructors.", Ex );
            }
            return CSharpMethods;
         }
      }

      public CSharpPropertyInfo[] OrderedLangProperties
      {
         get
         {
            int i = 0, j = 0;
            PropertyInfo[] Properties = null;
            CSharpPropertyInfo[] CSharpProperties = null;
            try
            {
               Properties = m_TypeObject.GetProperties( BindingFlags.NonPublic );
               CSharpProperties = new CSharpPropertyInfo[ Properties.Length ];
               PropertyComparer ICompProp = new PropertyComparer();
               Array.Sort( Properties, ICompProp );
               for ( ; i < Properties.Length; i++ )
               {
                  try
                  {
                     CSharpProperties[j] = new CSharpPropertyInfo( Properties[i] );
                     j++;
                  }
                  catch ( Exception /*ex*/ )
                  {
                     Console.WriteLine( "NativeProperty error {0}.{1}", Properties[i].ReflectedType.FullName, Properties[i].Name );
                  }
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in CSharpLangType.OrderedProperties", Ex );
            }
                     
            Console.WriteLine( "CSharp" );

            CSharpPropertyInfo[] np = new CSharpPropertyInfo[j];
            for ( i = 0; i < j; i++ )
               np[i] = CSharpProperties[i];
            return np;            
         }
      }

      public CSharpFieldInfo[] OrderedLangFields
      {
         get
         {
            FieldInfo[] Fields = null;
            CSharpFieldInfo[] CSharpFields = null;
            try
            {
               Fields = m_TypeObject.GetFields( BindingFlags.NonPublic );
               CSharpFields = new CSharpFieldInfo[ Fields.Length ];
               FieldComparer ICompField = new FieldComparer();
               Array.Sort( Fields, ICompField );
               for ( int i = 0; i < Fields.Length; i++ )
                  CSharpFields[i] = new CSharpFieldInfo( Fields[i] );
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in CSharpLangType.OrderedFields.", Ex );
            }
            return CSharpFields;
         }
      }

      public string HTMLDeclaration
      {
         override get { throw new NotSupportedException( "Not yet implemented." ); }
      }

      public string XMLDeclaration
      {
         override get { throw new NotSupportedException( "Not yet implemented." ); }
      }

      public string Access
      {
         get
         {
            if ( m_TypeObject.IsPublic )
               return "public ";
            else 
               return String.Empty;
         }
      }

      public string Attributes
      {
         get
         {
            StringBuilder s = new StringBuilder( m_ATTRIBUTE_STRING_SIZE );
         
            if ( m_TypeObject.IsAbstract && !m_TypeObject.IsInterface )
            {
               s.Append( "abstract " );
            }
            else if ( m_TypeObject.IsSealed && !m_TypeObject.IsValueType && !m_TypeObject.IsSubclassOf( typeof(Delegate) ) && !( m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) )
            {
               s.Append( "sealed " );
            }
            return s.ToString();
         }   
      }

      public string TypeCategory
      {
         get
         {
            string s = String.Empty;
            if ( m_TypeObject.IsSubclassOf( typeof( Delegate ) ) && !( m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) )
            {
               StringBuilder sb = new StringBuilder( "delegate " );
               MethodInfo m = m_TypeObject.GetMethod( "Invoke" );
               if ( m.ReturnType.Name.Equals( "Void" ) )
                  sb.Append( "void" );
               else
                  sb.Append( TranslateTypeName( m.ReturnType.Name ) );
               sb.Append( " " );
               sb.Append( m_TypeObject.Name );

               ParameterInfo[] p = m.GetParameters();
               Type[] param_types = new Type[p.Length];
               for ( int i = 0; i < param_types.Length; i++ )
                  param_types[i] = p[i].ParameterType;

               sb.Append( "(" );
               for ( int i = 0; i < p.Length; i++ )
               {        
                  sb.Append( String.Format( "{0} {1}", TranslateTypeName( param_types[i].Name ), p[i].Name ) );
                  if ( i + 1 < p.Length )
                     sb.Append( ", " );
               }
            
               sb.Append( ")" );
               s = sb.ToString();
            }        
            else if ( m_TypeObject.IsInterface )
               s = "interface";
            else if ( m_TypeObject.IsEnum )
               s = "enum";
            else if ( m_TypeObject.IsValueType )
               s = "struct";
            else 
               s = "class";
            return s;
         }
      }

      public string Extends
      {
         get
         {
            if (m_TypeObject.IsUnmanagedValueType)
               return String.Empty;
            if ( m_TypeObject.IsSubclassOf( typeof( Delegate ) ) && !( m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) )
               return String.Empty;
            if ( m_TypeObject.IsSubclassOf( typeof( Enum ) ) && !( m_TypeObject == typeof( Enum ) ) )
               return String.Empty;
            StringBuilder s = new StringBuilder();
                     
            bool found = false;

            // There is only a single parent class, if we are doing Object, then it has no parent, 
            // otherwise, if the parent is just object, we can ignore that, it is assumed.
            if ( m_TypeObject != typeof( Object ) && 
                 m_TypeObject.BaseType != typeof( Object ) && !m_TypeObject.IsInterface )
            {
               found = true;
               String temp = m_TypeObject.BaseType.Name;
               s.Append( String.Format( " : {0}", temp ) );
            }

            // Now we print the interfaces... there could be any number of these
            Hashtable hash = new Hashtable();
            for ( Type cur = m_TypeObject.BaseType; cur != null; cur = cur.BaseType )
            {
               Type[] Interfaces = cur.GetInterfaces();
               for ( int i = 0; i < Interfaces.Length; i++ )
               {
                  if ( !hash.ContainsKey( Interfaces[i] ) ) 
                     hash.Add( Interfaces[i], null );
               }
            }

            Type[] interfaces = m_TypeObject.GetInterfaces();
            ArrayList list = new ArrayList();
            for ( int i = 0; i < interfaces.Length; i++ )
            {
               if ( !hash.ContainsKey( interfaces[i] ) ) 
                  list.Add( interfaces[i] );
            }                          

            // interfaces = (Type[])list.ToArray( typeof( Type ) );
            interfaces = new Type[list.Count];
		      for ( int listIndex = 0; listIndex < list.Count; listIndex++ )
            {
			      interfaces[listIndex] = (Type)list[listIndex];
		      }

            // Type[] interfaces = m_TypeObject.GetInterfaces ();
            if ( interfaces.Length != 0 )
            {
               String temp = interfaces[0].Name;
               if ( !found )
                  s.Append( ": " );
               else
                  s.Append( ", " );
               s.Append( temp );
            }

            for ( int i = 1; i < interfaces.Length; i++ )
            {
               String temp = interfaces[i].Name;
               s.Append( String.Format( ", {0}", temp ) );
            }
            return s.ToString();
         }
      }

      public string TextDeclaration
      {
         override get
         {
            string m_Extends = String.Empty;
            string m_TypeName = String.Empty;

            if ( !( m_TypeObject.IsSubclassOf( typeof( Delegate ) ) && ! ( m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) ) ) 
            {
               m_Extends = Extends + ";";
               m_TypeName = " " + m_TypeObject.Name;
            }
            else
               m_TypeName += ";";

            string tmp = Access + Attributes + TypeCategory + m_TypeName;
            if ( m_TypeObject.IsEnum )
            {
               if ( Enum.GetUnderlyingType( m_TypeObject ) != typeof( Int32 ) )
                  tmp += " : " + TranslateTypeName( Enum.GetUnderlyingType( m_TypeObject ).ToString() );
            }
            tmp += m_Extends;
            return tmp;
         }
      }

   }

} // namespace LangUtil
