/*=====================================================================
  File:      LangEventInfo.cs

  Summary:   LangEventInfo implements or declares functionality common
         to subclasses that represent a particular language.

---------------------------------------------------------------------
  This file is part of the Microsoft NGWS SDK Code Samples.

  Copyright (C) 2000 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

namespace LangUtil {

using System;
using System.Reflection;
using System.Collections;

   public class LangEventInfo : LangMember
   {
      internal String m_Name = String.Empty;
      internal EventInfo m_ThisEventInfo = null;

      public LangEventInfo( EventInfo myEvent )
      {
         if ( myEvent == null )
            throw new NullReferenceException( "The EventInfo passed to the constructor is null." );

         // Determine method name and return type       
         m_Name = myEvent.Name;
         m_ThisEventInfo = myEvent;            
      }

      public Type ThisType
      {
         override get { return m_ThisEventInfo.ReflectedType; }
      }

      public string Attributes
      {
         override get { throw new NotSupportedException( "Attributes has not yet been implemented on this class." ); }
      }
   
      public EventInfo ThisEventInfo 
      {
         get { return m_ThisEventInfo; }
      }

      public override bool IsEvent()
      {      
         return true;
      }

      public String Name
      {
         override get { return m_Name; }
      }

   }

} // namespace LangUtil
