/*=====================================================================
  File:      NativeLAngType.cs

  Summary:   Brief summary of the file contents and purpose.

---------------------------------------------------------------------
  This file is part of the Microsoft NGWS SDK Code Samples.

  Copyright (C) 2000 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

namespace LangUtil {

using System;
using System.IO;
using System.Text;
using System.Reflection;
using System.Collections;

   public class NativeLangType : LangType
   {
      public NativeLangType( Type ThisType ) : base( ThisType )
      {
      }

      public NativeLangType( ParameterInfo ParamInfo ) : base( ParamInfo )
      {
      }

      public string TextDeclaration
      {
         override get
         {
            string typeName;
            Type t = null;
            typeName = m_TypeObject.Name;
                        
            try
            {
               t = m_TypeObject;
            }
            catch ( Exception e )
            {
               Console.WriteLine( "<p>{0}: Please try again. \"{1}\" is not a valid type name.  The syntax for the request is class.aspx?name=System.Type, where System.Type is the fully-qualified name of a type.</p>", 
                                  e.GetType().FullName, typeName );
            }

            if ( t == null )
            {
               Console.Write( "t is null." );
            }
            else
            {
               StringBuilder sb = new StringBuilder( (int)Math.Pow( 2, 10 ), Int32.MaxValue );
               sb.Append( Visibility( t ) );
               sb.Append( Accessibility( t ) );
               sb.Append( ImplementationAttributes( t ) );
               sb.Append( InteropAttributes( t ) );
               sb.Append( ReservedInternalUse( t ) );
               sb.Append( AdditionalSemantics( t ) );
               sb.Append( Semantics( t ) );
               sb.Append( t.Name );
               return sb.ToString();
            }
            return String.Empty;
         }
      }

      private string Visibility( Type t )
      {
         string result = "";

         if ( t.IsPublic )
            result = "tdPublic ";
         else
            result = "tdNotPublic ";

         return result;
      }

      private string Accessibility( Type t )
      {
         string result = "";
         if ( t.IsNestedPrivate )
            result = "tdNestedPrivate ";
         if ( t.IsNestedFamily )
            result = "tdNestedFamily ";
         if ( t.IsNestedAssembly )
            result = "tdNestedAssembly ";
         if ( t.IsNestedFamANDAssem )
            result = "tdNestedFamANDAssem ";
         if ( t.IsNestedFamORAssem )
            result = "tdNestedFamORAssem ";
         return result;
      }

      private string Semantics( Type t )
      {
         string result = "";
         if ( t.IsClass ) 
            result = "tdClass ";
         if ( t.IsInterface ) 
            result = "tdInterface ";
         if ( t.IsValueType ) 
            result = "tdValueType ";
         if ( t.IsUnmanagedValueType ) 
            result = "tdUnmanagedValueType ";
         return result;
      }

      private string AdditionalSemantics( Type t )
      {
         string result = "";
         if ( t.IsAbstract ) 
            result = "tdAbstract ";
         if ( t.IsSealed ) 
            result = "tdSealed ";
         if ( t.IsEnum ) 
            result = "tdEnum ";
         if ( t.IsSpecialName ) 
            result = "tdSpecialName ";
         // if (t.IsRTSpecialName) result = "tdRTSpecialName ";
         return result;
      }     

      private string ImplementationAttributes( Type t )
      {
         string result = "";
         if ( t.IsSerializable ) 
            result = "tdSerializable ";
         if ( t.IsContextful ) 
            result = "tdContextful ";
         if ( t.IsMarshalByRef ) 
            result = "tdMarshalByRef ";
         return result;
      }

      private string InteropAttributes( Type t )
      {
         string result = "";
         if ( t.IsAnsiClass ) 
            result = "tdAnsiClass ";
         if ( t.IsUnicodeClass ) 
            result = "tdAutoClass ";
         if ( t.IsAutoClass ) 
            result = "tdAutoClass ";
         return result;
      }

      private string ReservedInternalUse( Type t )
      {
         string result = "";
         if ( t.IsImport ) 
            result = "tdImport ";
         // if (t.IsReserved1) result = "tdReserved1 ";
         return result;
      }

      public string HTMLDeclaration
      {
         override get { throw new NotSupportedException( "Not yet implemented." ); }
      }

      public string XMLDeclaration
      {
         override get { throw new NotSupportedException( "Not yet implemented." ); }
      }

      public NativeConstructorInfo[] OrderedNativeConstructors
      {
         get
         {
            ConstructorInfo[] Constructors = null;
            NativeConstructorInfo[] NativeConstructors = null;

            try
            {
               Constructors = m_TypeObject.GetConstructors( BindingFlags.LookupAll );
               NativeConstructors = new NativeConstructorInfo[Constructors.Length];
               ConstructorComparer ICompConst = new ConstructorComparer();
               Array.Sort( Constructors, ICompConst );
               for ( int i = 0; i < Constructors.Length; i++ )
               {
                  NativeConstructors[i] = new NativeConstructorInfo( Constructors[i] );
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in NativeLangType.OrderedNativeConstructors.", Ex );
            }
            return NativeConstructors;
         }
      }

      public NativeMethodInfo[] OrderedNativeMethods
      {
         get
         {
            MethodInfo [] Methods = null;
            NativeMethodInfo[] NativeMethods = null;

            try
            {
               Methods = m_TypeObject.GetMethods( BindingFlags.LookupAll );
               NativeMethods = new NativeMethodInfo[Methods.Length];

               // Order the member categories
               MethodComparer ICompMet = new MethodComparer();
               Array.Sort( Methods, ICompMet );

               for ( int i = 0; i < Methods.Length; i++ )
               {
                  NativeMethods[i] = new NativeMethodInfo( Methods[i] );
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in NativeLangType.OrderedNativeMethods.", Ex );
            }
            return NativeMethods;
         }
      }

      public NativePropertyInfo[] OrderedNativeProperties
      {
         get
         {
            int i = 0, j = 0;
            PropertyInfo[] Properties = null;
            NativePropertyInfo[] NativeProperties = null;
            try
            {
               Properties = m_TypeObject.GetProperties( BindingFlags.LookupAll );
               NativeProperties = new NativePropertyInfo[Properties.Length];
               PropertyComparer ICompProp = new PropertyComparer();
               Array.Sort( Properties, ICompProp );
               for ( ; i < Properties.Length; i++ )
               {
                  try
                  {
                     NativeProperties[j] = new NativePropertyInfo( Properties[i] );
                     j++;
                  }
                  catch ( Exception /*ex*/ )
                  {
                     // Console.WriteLine( "NativeProperty error {0}.{1}", Properties[i].ReflectedType.FullName, Properties[i].Name );
                  }
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in NativeLangType.OrderedNativeProperties:" 
                                    + Ex.TargetSite.Name, Ex );
            }

            NativePropertyInfo[] np = new NativePropertyInfo[j];
            for ( i = 0; i < j; i++ )
               np[i] = NativeProperties[i];
            return np;
         }
      }

      public NativeFieldInfo[] OrderedNativeFields
      {
         get
         {
            FieldInfo[] Fields = null;
            NativeFieldInfo[] NativeFields = null;
            try
            {
               Fields = m_TypeObject.GetFields( BindingFlags.LookupAll );
               NativeFields = new NativeFieldInfo[Fields.Length];
               FieldComparer ICompField = new FieldComparer();
               Array.Sort( Fields, ICompField );
               for ( int i = 0; i < Fields.Length; i++ )
               {
                  NativeFields[i] = new NativeFieldInfo( Fields[i] );
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in NativeLangType.OrderedNativeFields.", Ex );
            }
            return NativeFields;
         }
      }

      public NativeEventInfo[] OrderedNativeEvents
      {
         get
         {
            EventInfo[] Events = null;
            NativeEventInfo[] NativeEvents = null;

            try
            {
               Events = m_TypeObject.GetEvents( BindingFlags.LookupAll );
               NativeEvents = new NativeEventInfo[Events.Length];
               FieldComparer ICompField = new FieldComparer();
               Array.Sort( Events, ICompField );

               for ( int i = 0; i < Events.Length; i++ )
               {
                  NativeEvents[i] = new NativeEventInfo( Events[i] );
               }
            }
            catch( Exception Ex )
            {
               throw new Exception( "Occurred in NativeLangType.OrderedNativeEvents.", Ex );
            }
            return NativeEvents;
         }
      }
   }

} // namespace LangUtil
